searchData={"items":[{"type":"module","title":"Upward","doc":"Upward is a library to assist with hot-code upgrades with Elixir releases.\n\nThis module manages the building and installing of releases.\n\nMost functions print messages to the console as they are executed.","ref":"Upward.html"},{"type":"function","title":"Upward.auto_appup/2","doc":"Automatically generate an appup file for a patch release based on the previous release in your build directory.\n\nThis should be added as a step in your release steps.\n```elixir\ndef releases do\n  [\n    my_app: [\n      include_executables_for: [:unix],\n      steps: [:assemble, &Upward.auto_appup/1 :tar]\n    ]\n  ]\nend\n```","ref":"Upward.html#auto_appup/2"},{"type":"function","title":"Upward.downgrade/0","doc":"Downgrade or rollback to the previous patch release.\n\nThis will not downgrade to before x.y.0\n\nRun in your release directory using:\n```bash\nbin/my_app rpc 'Upward.downgrade()'\n```","ref":"Upward.html#downgrade/0"},{"type":"function","title":"Upward.install/1","doc":"Install a new release.\n\nYou need to extract the tarball generated by the release into your release directory before running this command.\nThere needs to be an appup file generated between the current running version and the version you select to install.\n\nRun in your release directory using:\n```bash\nbin/my_app rpc 'Upward.install(\"1.0.1\")'\n```","ref":"Upward.html#install/1"},{"type":"function","title":"Upward.releases/0","doc":"Prints a list of all the current releases along with their status.\n\nSee also `Upward.Releases.releases/0`","ref":"Upward.html#releases/0"},{"type":"function","title":"Upward.upgrade/0","doc":"Upgrade to the next patch release.\n\nYou must have already extracted the tarball generated by the release into your release directory before running this command.\n\nThere needs to be an appup file generated between the current running version and the next patch version.\nThis would be automatically generated within your release by using `&Upward.auto_appup/1` in your release steps.\n\nRun in your release directory using:\n```bash\nbin/my_app rpc 'Upward.upgrade()'\n```","ref":"Upward.html#upgrade/0"},{"type":"module","title":"Upward.Appup","doc":"This module is responsible for generating appup files for changes between two versions of an application.","ref":"Upward.Appup.html"},{"type":"type","title":"Upward.Appup.app/0","doc":"","ref":"Upward.Appup.html#t:app/0"},{"type":"type","title":"Upward.Appup.appup_ver/0","doc":"","ref":"Upward.Appup.html#t:appup_ver/0"},{"type":"type","title":"Upward.Appup.change/0","doc":"","ref":"Upward.Appup.html#t:change/0"},{"type":"type","title":"Upward.Appup.dep_mods/0","doc":"","ref":"Upward.Appup.html#t:dep_mods/0"},{"type":"type","title":"Upward.Appup.instruction/0","doc":"","ref":"Upward.Appup.html#t:instruction/0"},{"type":"function","title":"Upward.Appup.make/6","doc":"Generate an appup file for changes between two versions of an application.","ref":"Upward.Appup.html#make/6"},{"type":"type","title":"Upward.Appup.version_str/0","doc":"","ref":"Upward.Appup.html#t:version_str/0"},{"type":"behaviour","title":"Upward.Appup.Transform","doc":"A transform is an appup compilation pass which receives a list of appup instructions,\nalong with metadata about those instructions, such as the source application,\nthe source and target versions involved, and an optional list of configuration options\nfor the transform.\n\nThe job of a transform is to, well, apply a transformation to the instruction set, in\norder to accomplish some objective that one desires to be automated. A trivial example\nof one such transform would be a transform which ensures the purge mode is set to `:soft_purge`\nfor all `:update` instructions. To see an example of such a transform, look in `test/support/purge_transform.ex`","ref":"Upward.Appup.Transform.html"},{"type":"type","title":"Upward.Appup.Transform.app/0","doc":"","ref":"Upward.Appup.Transform.html#t:app/0"},{"type":"function","title":"Upward.Appup.Transform.down/5","doc":"Applies all transforms against the current downgrade instruction.\n\nAdditional information required as arguments and passed to transforms are\nthe app the instruction applies to, and the source and target versions involved.","ref":"Upward.Appup.Transform.html#down/5"},{"type":"callback","title":"Upward.Appup.Transform.down/5","doc":"","ref":"Upward.Appup.Transform.html#c:down/5"},{"type":"type","title":"Upward.Appup.Transform.instruction/0","doc":"","ref":"Upward.Appup.Transform.html#t:instruction/0"},{"type":"type","title":"Upward.Appup.Transform.options/0","doc":"","ref":"Upward.Appup.Transform.html#t:options/0"},{"type":"type","title":"Upward.Appup.Transform.transform/0","doc":"","ref":"Upward.Appup.Transform.html#t:transform/0"},{"type":"function","title":"Upward.Appup.Transform.up/5","doc":"Applies all transforms against the current upgrade instruction.\n\nAdditional information required as arguments and passed to transforms are\nthe app the instruction applies to, and the source and target versions involved.","ref":"Upward.Appup.Transform.html#up/5"},{"type":"callback","title":"Upward.Appup.Transform.up/5","doc":"","ref":"Upward.Appup.Transform.html#c:up/5"},{"type":"type","title":"Upward.Appup.Transform.version/0","doc":"","ref":"Upward.Appup.Transform.html#t:version/0"},{"type":"module","title":"Upward.Config","doc":"A utility module for managing the configuration of an application.","ref":"Upward.Config.html"},{"type":"function","title":"Upward.Config.diff/2","doc":"Returns a tuple with the new config, the changed config, and the removed config.\nnew - config with a key that is not in the previous config\nchanged - config with a key that is in the previous config and has a different value\nremoved - list of keys that are in the previous config but not in the new config","ref":"Upward.Config.html#diff/2"},{"type":"module","title":"Upward.Releases","doc":"A utility module for managing releases.","ref":"Upward.Releases.html"},{"type":"function","title":"Upward.Releases.app_name/0","doc":"Get the name of the running application.","ref":"Upward.Releases.html#app_name/0"},{"type":"function","title":"Upward.Releases.current_version/0","doc":"Get the current version of the running application.","ref":"Upward.Releases.html#current_version/0"},{"type":"function","title":"Upward.Releases.install_release/1","doc":"Install a release.\n\nThis will ignore an already installed release.","ref":"Upward.Releases.html#install_release/1"},{"type":"function","title":"Upward.Releases.make_permanent/1","doc":"Make a release permanent.","ref":"Upward.Releases.html#make_permanent/1"},{"type":"function","title":"Upward.Releases.make_releases_file/3","doc":"Make a RELEASES file for a given version.\n\nThis is only done for versions x.y.0 and creates a RELEASES file to be included with a release.","ref":"Upward.Releases.html#make_releases_file/3"},{"type":"function","title":"Upward.Releases.next_version/1","doc":"Get the next patch version (of the running application).","ref":"Upward.Releases.html#next_version/1"},{"type":"function","title":"Upward.Releases.previous_version/0","doc":"Get the previous installed patch version of the running application.","ref":"Upward.Releases.html#previous_version/0"},{"type":"function","title":"Upward.Releases.releases/0","doc":"Get a list of all the releases along with their status.","ref":"Upward.Releases.html#releases/0"},{"type":"function","title":"Upward.Releases.remove_releases_file/1","doc":"Remove the RELEASES file (from the build directory) so it is not included with a patch release.","ref":"Upward.Releases.html#remove_releases_file/1"},{"type":"function","title":"Upward.Releases.set_unpacked/1","doc":"This is as part of a version install and assumes that the release tarball has been extracted into the release directory.","ref":"Upward.Releases.html#set_unpacked/1"},{"type":"module","title":"Upward.Relup","doc":"A utility module for managing relup files.","ref":"Upward.Relup.html"},{"type":"function","title":"Upward.Relup.make/2","doc":"Make a relup file between two versions.","ref":"Upward.Relup.html#make/2"},{"type":"module","title":"Upward.Transformers.PhoenixLiveViewTransformer","doc":"An appup transformer for Phoenix LiveView.\n\nIf this transformer is included, it will append instructions to have all\nPhoenix LiveView channels updated.\n\nIf you want your LiveView to be updated during an upgrade/downgrade,\nyou should implement `code_change/3` in your LiveView.\n\n```elixir\ndef code_change(_old_vsn, socket, _extra) do\n  {:ok, socket}\nend\n```\n\n`old_vsn` will be the previous version of the Channel (**not the LiveView**) and `{down: old_vsn}` during a downgrade.\nBecause the version is the version of the Channel, you should implement your own version tracking within the socket private or assigns.\n\nAdd this transform to the auto_appup step in your release config\n\n```elixir\ndef releases do\n  [\n    my_app: [\n      include_executables_for: [:unix],\n      steps: [\n        :assemble,\n        &Upward.auto_appup(&1, transforms: [Upward.Transformers.PhoenixLiveViewTransformer]),\n        :tar\n      ]\n    ]\n  ]\nend\n```","ref":"Upward.Transformers.PhoenixLiveViewTransformer.html"},{"type":"function","title":"Upward.Transformers.PhoenixLiveViewTransformer.down/5","doc":"","ref":"Upward.Transformers.PhoenixLiveViewTransformer.html#down/5"},{"type":"function","title":"Upward.Transformers.PhoenixLiveViewTransformer.up/5","doc":"","ref":"Upward.Transformers.PhoenixLiveViewTransformer.html#up/5"},{"type":"module","title":"Upward.Utils","doc":"General utility functions.","ref":"Upward.Utils.html"},{"type":"function","title":"Upward.Utils.is_base_patch?/1","doc":"Checks if a version is a base patch version.\n\nBase patch versions are versions that have a patch number of 0.\n\nExamples:\n    Upward.Utils.is_base_patch?(\"1.0.0\")\n    iex> true\n\n    Upward.Utils.is_base_patch?(\"1.0.1\")\n    iex> false","ref":"Upward.Utils.html#is_base_patch?/1"},{"type":"function","title":"Upward.Utils.parse_version/1","doc":"Parses a version string into a Version struct.\n\nExamples:\n    Upward.Utils.parse_version(\"1.0.0\")\n    iex> %Version{major: 1, minor: 0, patch: 0}","ref":"Upward.Utils.html#parse_version/1"},{"type":"function","title":"Upward.Utils.patch_release?/2","doc":"Checks if the difference between two versions is a patch release.\n\nExamples:\n    Upward.Utils.patch_release?(\"1.0.0\", \"1.0.1\")\n    iex> true\n\n    Upward.Utils.patch_release?(\"1.0.0\", \"1.1.0\")\n    iex> false","ref":"Upward.Utils.html#patch_release?/2"},{"type":"function","title":"Upward.Utils.previous_release_path/2","doc":"Finds the previous release path within the build directory.","ref":"Upward.Utils.html#previous_release_path/2"},{"type":"exception","title":"Upward.Appup.TransformError","doc":"This error is raised when an appup transformation results in an\ninvalid appup instruction or instruction set.","ref":"Upward.Appup.TransformError.html"},{"type":"extras","title":"Upward","doc":"# Upward\n\nAn opinionated library to assist with hot-code upgrades with Elixir releases","ref":"readme.html"},{"type":"extras","title":"Installation - Upward","doc":"Add `:upward` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:upward, \"~> 0.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Philosophy - Upward","doc":"Elixir, built on top of Erlang OTP, has an incredibly powerful feature of being able to hot-swap code on a running system.\nLike all the powerful features that come with OTP, it has the potential to be very complex and difficult to understand.\n\nUpward is built to provide a simple approach to hot-code upgrades, and purposefully limits hot-code upgrades to patch releases.\n\nThe idea being that you are able to release early and release often, but when you make significant changes, like adding or upgrading dependencies, altering your supervision tree, or similar, you create a new minor or major release.\n\nWhen you do a patch release, Upward will automatically generate an appup file with the changes needed to upgrade from the previous release to the new one.\nWhen you do a minor or major release, Upward will not generate an appup file, but will instead create a new RELEASES file which will service as the bases for future patch releases.\nYou should then restart your system using an appropriate strategy, like a rolling restart or a blue/green deployment.\n\nYou will be able to do hot-rollbacks through patch releases, back to the last minor or major release.","ref":"readme.html#philosophy"},{"type":"extras","title":"Phoenix LiveView - Upward","doc":"Without hot-code upgrades, Phoenix LiveView applications will have to reconnect the socket and regenerate state for every update. Clients will receive reconnection messages and similar UI irritations. This is completely unnecessary if you’re making simple changes to your codebase.\n\nBy using upward, you can minimise discruptions, especially for long-running sessions.","ref":"readme.html#phoenix-liveview"},{"type":"extras","title":"Demo - Upward","doc":"You can see a demo of Upward in action [here](https://upward.andrewtimberlake.com).","ref":"readme.html#demo"},{"type":"extras","title":"Usage - Upward","doc":"Add `&Upward.auto_appup/1` to your release steps in `mix.exs`.\n\n```elixir\n  def releases do\n    [\n      my_app: [\n        include_executables_for: [:unix],\n        steps: [:assemble, &Upward.auto_appup/1, :tar]\n      ]\n    ]\n  end\n```","ref":"readme.html#usage"},{"type":"extras","title":"Usage with Phoenix LiveView - Upward","doc":"Phoenix LiveView channels can be updated during a patch release by implementing the `code_change/3` callback in your LiveView.\nBut, your LiveView runs within the context of a Phoenix Channel, so code_change is only called if the Channel is set to update. In order to do that, we need to add an update instruction for the channel within the app up.\n\nIf you want `code_change/3` to be called on your liveviews during an upgrade, then add `Upward.Transformers.PhoenixLiveViewTransformer` to the auto_appup step in your release config.\n\nSee instructions at `Upward.Transformers.PhoenixLiveViewTransformer` for more information.","ref":"readme.html#usage-with-phoenix-liveview"},{"type":"extras","title":"Upgrading your app - Upward","doc":"For each patch release, you will need run the upgrade from within your app using `Upward.upgrade/0`.\n\nHere are the release steps:\n1. `mix release`\n2. Copy the tarball to the target machine\n3. Extract the tarball to the release directory\n4. Run `Upward.upgrade/0`\n   - `$ bin/my_app rpc 'Upward.upgrade()'`\n\nTo rollback to a previous version, run `Upward.downgrade/0`.\n\nYou can also run `Upward.install/1` with a specific version to install a specific version. It is up to you to ensure that the version you select is the next version up or down from the current running version.","ref":"readme.html#upgrading-your-app"},{"type":"extras","title":"Author - Upward","doc":"[Andrew Timberlake](https://andrewtimberlake.com)\nIf you need help with your Elixir or Phoenix project, [I'm available for hire](https://andrewtimberlake.com/hire).","ref":"readme.html#author"},{"type":"extras","title":"Versioning - Upward","doc":"This library follows [Semantic Versioning](https://semver.org/), and expects you to follow the same ;-).","ref":"readme.html#versioning"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}